#ifdef HAVE_GLX
	if (!strncmp(name, "GLX_", 4)) {
		i(GLX_USE_GL)
		i(GLX_BUFFER_SIZE)
		i(GLX_LEVEL)
		i(GLX_RGBA)
		i(GLX_DOUBLEBUFFER)
		i(GLX_STEREO)
		i(GLX_AUX_BUFFERS)
		i(GLX_RED_SIZE)
		i(GLX_GREEN_SIZE)
		i(GLX_BLUE_SIZE)
		i(GLX_ALPHA_SIZE)
		i(GLX_DEPTH_SIZE)
		i(GLX_STENCIL_SIZE)
		i(GLX_ACCUM_RED_SIZE)
		i(GLX_ACCUM_GREEN_SIZE)
		i(GLX_ACCUM_BLUE_SIZE)
		i(GLX_ACCUM_ALPHA_SIZE)
#ifdef GLX_EXT_visual_info
		i(GLX_X_VISUAL_TYPE_EXT)
		i(GLX_TRANSPARENT_TYPE_EXT)
		i(GLX_TRANSPARENT_INDEX_VALUE_EXT)
		i(GLX_TRANSPARENT_RED_VALUE_EXT)
		i(GLX_TRANSPARENT_GREEN_VALUE_EXT)
		i(GLX_TRANSPARENT_BLUE_VALUE_EXT)
		i(GLX_TRANSPARENT_ALPHA_VALUE_EXT)
#endif
	}
	if (1) {
		i(X_PROTOCOL)
		i(X_PROTOCOL_REVISION)
		i(NoEventMask)
		i(KeyPressMask)
		i(KeyReleaseMask)
		i(ButtonPressMask)
		i(ButtonReleaseMask)
		i(EnterWindowMask)
		i(LeaveWindowMask)
		i(PointerMotionMask)
		i(PointerMotionHintMask)
		i(Button1MotionMask)
		i(Button2MotionMask)
		i(Button3MotionMask)
		i(Button4MotionMask)
		i(Button5MotionMask)
		i(ButtonMotionMask)
		i(KeymapStateMask)
		i(ExposureMask)
		i(VisibilityChangeMask)
		i(StructureNotifyMask)
		i(ResizeRedirectMask)
		i(SubstructureNotifyMask)
		i(SubstructureRedirectMask)
		i(FocusChangeMask)
		i(PropertyChangeMask)
		i(ColormapChangeMask)
		i(OwnerGrabButtonMask)
		i(KeyPress)
		i(KeyRelease)
		i(ButtonPress)
		i(ButtonRelease)
		i(MotionNotify)
		i(EnterNotify)
		i(LeaveNotify)
		i(FocusIn)
		i(FocusOut)
		i(KeymapNotify)
		i(Expose)
		i(GraphicsExpose)
		i(NoExpose)
		i(VisibilityNotify)
		i(CreateNotify)
		i(DestroyNotify)
		i(UnmapNotify)
		i(MapNotify)
		i(MapRequest)
		i(ReparentNotify)
		i(ConfigureNotify)
		i(ConfigureRequest)
		i(GravityNotify)
		i(ResizeRequest)
		i(CirculateNotify)
		i(CirculateRequest)
		i(PropertyNotify)
		i(SelectionClear)
		i(SelectionRequest)
		i(SelectionNotify)
		i(ColormapNotify)
		i(ClientMessage)
		i(MappingNotify)
		i(LASTEvent)
		i(ShiftMask)
		i(LockMask)
		i(ControlMask)
		i(Mod1Mask)
		i(Mod2Mask)
		i(Mod3Mask)
		i(Mod4Mask)
		i(Mod5Mask)
		i(ShiftMapIndex)
		i(LockMapIndex)
		i(ControlMapIndex)
		i(Mod1MapIndex)
		i(Mod2MapIndex)
		i(Mod3MapIndex)
		i(Mod4MapIndex)
		i(Mod5MapIndex)
		i(Button1Mask)
		i(Button2Mask)
		i(Button3Mask)
		i(Button4Mask)
		i(Button5Mask)
		i(AnyModifier)
		i(Button1)
		i(Button2)
		i(Button3)
		i(Button4)
		i(Button5)
		i(NotifyNormal)
		i(NotifyGrab)
		i(NotifyUngrab)
		i(NotifyWhileGrabbed)
		i(NotifyHint)
		i(NotifyAncestor)
		i(NotifyVirtual)
		i(NotifyInferior)
		i(NotifyNonlinear)
		i(NotifyNonlinearVirtual)
		i(NotifyPointer)
		i(NotifyPointerRoot)
		i(NotifyDetailNone)
		i(VisibilityUnobscured)
		i(VisibilityPartiallyObscured)
		i(VisibilityFullyObscured)
		i(PlaceOnTop)
		i(PlaceOnBottom)
		i(FamilyInternet)
		i(FamilyDECnet)
		i(FamilyChaos)
		i(PropertyNewValue)
		i(PropertyDelete)
		i(ColormapUninstalled)
		i(ColormapInstalled)
		i(GrabModeSync)
		i(GrabModeAsync)
		i(GrabSuccess)
		i(AlreadyGrabbed)
		i(GrabInvalidTime)
		i(GrabNotViewable)
		i(GrabFrozen)
		i(AsyncPointer)
		i(SyncPointer)
		i(ReplayPointer)
		i(AsyncKeyboard)
		i(SyncKeyboard)
		i(ReplayKeyboard)
		i(AsyncBoth)
		i(SyncBoth)
		i(StaticGray)
		i(GrayScale)
		i(StaticColor)
		i(PseudoColor)
		i(TrueColor)
		i(DirectColor)
	}
	if (0) {
	}
	else
#endif /* def HAVE_GLX */
